# -*- coding: utf-8 -*-

# Copyright (c) 2019, Trevor Highfill <trevor.highfill@outlook.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r'''
  name: diy
  type: stdout
  short_description: Customize the output
  version_added: 0.2.0
  description:
    - Callback plugin that allows you to supply your own custom callback templates to be output.
  author: Trevor Highfill (@theque5t)
  extends_documentation_fragment:
    - default_callback
  notes:
    - Uses the P(ansible.builtin.default#callback) callback plugin output when a custom callback V(message(msg\)) is not provided.
    - Makes the callback event data available via the C(ansible_callback_diy) dictionary, which can be used in the templating context for the options.
      The dictionary is only available in the templating context for the options. It is not a variable that is available via the other
      various execution contexts, such as playbook, play, task etc.
    - Options being set by their respective variable input can only be set using the variable if the variable was set in a context that is available to the
      respective callback.
      Use the C(ansible_callback_diy) dictionary to see what is available to a callback. Additionally, C(ansible_callback_diy.top_level_var_names) will output
      the top level variable names available to the callback.
    - Each option value is rendered as a template before being evaluated. This allows for the dynamic usage of an option. For example,
      C("{{ 'yellow' if ansible_callback_diy.result.is_changed else 'bright green' }}")
    - "**Condition** for all C(msg) options:
                    if value C(is None or omit),
                    then the option is not being used.
       **Effect**: use of the C(default) callback plugin for output"
    - "**Condition** for all C(msg) options:
                    if value C(is not None and not omit and length is not greater than 0),
                    then the option is being used without output.
       **Effect**: suppress output"
    - "**Condition** for all C(msg) options:
                    if value C(is not None and not omit and length is greater than 0),
                    then the option is being used with output.
       **Effect**: render value as template and output"
    - "Valid color values: V(black), V(bright gray), V(blue), V(white), V(green), V(bright blue), V(cyan), V(bright green), V(red), V(bright cyan),
      V(purple), V(bright red), V(yellow), V(bright purple), V(dark gray), V(bright yellow), V(magenta), V(bright magenta), V(normal)"
  seealso:
    - name: default – default Ansible screen output
      description: The official documentation on the B(default) callback plugin.
      link: https://docs.ansible.com/ansible/latest/plugins/callback/default.html
  requirements:
    - set as stdout_callback in configuration
  options:
    on_any_msg:
      description: Output to be used for callback on_any.
      ini:
        - section: callback_diy
          key: on_any_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_ON_ANY_MSG
      vars:
        - name: ansible_callback_diy_on_any_msg
      type: str

    on_any_msg_color:
      description:
        - Output color to be used for O(on_any_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: on_any_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_ON_ANY_MSG_COLOR
      vars:
        - name: ansible_callback_diy_on_any_msg_color
      type: str

    runner_on_failed_msg:
      description: Output to be used for callback runner_on_failed.
      ini:
        - section: callback_diy
          key: runner_on_failed_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_FAILED_MSG
      vars:
        - name: ansible_callback_diy_runner_on_failed_msg
      type: str

    runner_on_failed_msg_color:
      description:
        - Output color to be used for O(runner_on_failed_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: runner_on_failed_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_FAILED_MSG_COLOR
      vars:
        - name: ansible_callback_diy_runner_on_failed_msg_color
      type: str

    runner_on_ok_msg:
      description: Output to be used for callback runner_on_ok.
      ini:
        - section: callback_diy
          key: runner_on_ok_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_OK_MSG
      vars:
        - name: ansible_callback_diy_runner_on_ok_msg
      type: str

    runner_on_ok_msg_color:
      description:
        - Output color to be used for O(runner_on_ok_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: runner_on_ok_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_OK_MSG_COLOR
      vars:
        - name: ansible_callback_diy_runner_on_ok_msg_color
      type: str

    runner_on_skipped_msg:
      description: Output to be used for callback runner_on_skipped.
      ini:
        - section: callback_diy
          key: runner_on_skipped_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_SKIPPED_MSG
      vars:
        - name: ansible_callback_diy_runner_on_skipped_msg
      type: str

    runner_on_skipped_msg_color:
      description:
        - Output color to be used for O(runner_on_skipped_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: runner_on_skipped_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_SKIPPED_MSG_COLOR
      vars:
        - name: ansible_callback_diy_runner_on_skipped_msg_color
      type: str

    runner_on_unreachable_msg:
      description: Output to be used for callback runner_on_unreachable.
      ini:
        - section: callback_diy
          key: runner_on_unreachable_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_UNREACHABLE_MSG
      vars:
        - name: ansible_callback_diy_runner_on_unreachable_msg
      type: str

    runner_on_unreachable_msg_color:
      description:
        - Output color to be used for O(runner_on_unreachable_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: runner_on_unreachable_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_UNREACHABLE_MSG_COLOR
      vars:
        - name: ansible_callback_diy_runner_on_unreachable_msg_color
      type: str

    playbook_on_start_msg:
      description: Output to be used for callback playbook_on_start.
      ini:
        - section: callback_diy
          key: playbook_on_start_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_START_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_start_msg
      type: str

    playbook_on_start_msg_color:
      description:
        - Output color to be used for O(playbook_on_start_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_start_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_START_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_start_msg_color
      type: str

    playbook_on_notify_msg:
      description: Output to be used for callback playbook_on_notify.
      ini:
        - section: callback_diy
          key: playbook_on_notify_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NOTIFY_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_notify_msg
      type: str

    playbook_on_notify_msg_color:
      description:
        - Output color to be used for O(playbook_on_notify_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_notify_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NOTIFY_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_notify_msg_color
      type: str

    playbook_on_no_hosts_matched_msg:
      description: Output to be used for callback playbook_on_no_hosts_matched.
      ini:
        - section: callback_diy
          key: playbook_on_no_hosts_matched_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NO_HOSTS_MATCHED_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_no_hosts_matched_msg
      type: str

    playbook_on_no_hosts_matched_msg_color:
      description:
        - Output color to be used for O(playbook_on_no_hosts_matched_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_no_hosts_matched_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NO_HOSTS_MATCHED_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_no_hosts_matched_msg_color
      type: str

    playbook_on_no_hosts_remaining_msg:
      description: Output to be used for callback playbook_on_no_hosts_remaining.
      ini:
        - section: callback_diy
          key: playbook_on_no_hosts_remaining_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NO_HOSTS_REMAINING_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_no_hosts_remaining_msg
      type: str

    playbook_on_no_hosts_remaining_msg_color:
      description:
        - Output color to be used for O(playbook_on_no_hosts_remaining_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_no_hosts_remaining_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NO_HOSTS_REMAINING_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_no_hosts_remaining_msg_color
      type: str

    playbook_on_task_start_msg:
      description: Output to be used for callback playbook_on_task_start.
      ini:
        - section: callback_diy
          key: playbook_on_task_start_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_TASK_START_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_task_start_msg
      type: str

    playbook_on_task_start_msg_color:
      description:
        - Output color to be used for O(playbook_on_task_start_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_task_start_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_TASK_START_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_task_start_msg_color
      type: str

    playbook_on_handler_task_start_msg:
      description: Output to be used for callback playbook_on_handler_task_start.
      ini:
        - section: callback_diy
          key: playbook_on_handler_task_start_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_HANDLER_TASK_START_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_handler_task_start_msg
      type: str

    playbook_on_handler_task_start_msg_color:
      description:
        - Output color to be used for O(playbook_on_handler_task_start_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_handler_task_start_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_HANDLER_TASK_START_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_handler_task_start_msg_color
      type: str

    playbook_on_vars_prompt_msg:
      description: Output to be used for callback playbook_on_vars_prompt.
      ini:
        - section: callback_diy
          key: playbook_on_vars_prompt_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_VARS_PROMPT_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_vars_prompt_msg
      type: str

    playbook_on_vars_prompt_msg_color:
      description:
        - Output color to be used for O(playbook_on_vars_prompt_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_vars_prompt_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_VARS_PROMPT_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_vars_prompt_msg_color
      type: str

    playbook_on_play_start_msg:
      description: Output to be used for callback playbook_on_play_start.
      ini:
        - section: callback_diy
          key: playbook_on_play_start_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_PLAY_START_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_play_start_msg
      type: str

    playbook_on_play_start_msg_color:
      description:
        - Output color to be used for O(playbook_on_play_start_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_play_start_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_PLAY_START_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_play_start_msg_color
      type: str

    playbook_on_stats_msg:
      description: Output to be used for callback playbook_on_stats.
      ini:
        - section: callback_diy
          key: playbook_on_stats_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_STATS_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_stats_msg
      type: str

    playbook_on_stats_msg_color:
      description:
        - Output color to be used for O(playbook_on_stats_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_stats_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_STATS_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_stats_msg_color
      type: str

    on_file_diff_msg:
      description: Output to be used for callback on_file_diff.
      ini:
        - section: callback_diy
          key: on_file_diff_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_ON_FILE_DIFF_MSG
      vars:
        - name: ansible_callback_diy_on_file_diff_msg
      type: str

    on_file_diff_msg_color:
      description:
        - Output color to be used for O(on_file_diff_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: on_file_diff_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_ON_FILE_DIFF_MSG_COLOR
      vars:
        - name: ansible_callback_diy_on_file_diff_msg_color
      type: str

    playbook_on_include_msg:
      description: Output to be used for callback playbook_on_include.
      ini:
        - section: callback_diy
          key: playbook_on_include_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_INCLUDE_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_include_msg
      type: str

    playbook_on_include_msg_color:
      description:
        - Output color to be used for O(playbook_on_include_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_include_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_INCLUDE_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_include_msg_color
      type: str

    runner_item_on_ok_msg:
      description: Output to be used for callback runner_item_on_ok.
      ini:
        - section: callback_diy
          key: runner_item_on_ok_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_OK_MSG
      vars:
        - name: ansible_callback_diy_runner_item_on_ok_msg
      type: str

    runner_item_on_ok_msg_color:
      description:
        - Output color to be used for O(runner_item_on_ok_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: runner_item_on_ok_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_OK_MSG_COLOR
      vars:
        - name: ansible_callback_diy_runner_item_on_ok_msg_color
      type: str

    runner_item_on_failed_msg:
      description: Output to be used for callback runner_item_on_failed.
      ini:
        - section: callback_diy
          key: runner_item_on_failed_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_FAILED_MSG
      vars:
        - name: ansible_callback_diy_runner_item_on_failed_msg
      type: str

    runner_item_on_failed_msg_color:
      description:
        - Output color to be used for O(runner_item_on_failed_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: runner_item_on_failed_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_FAILED_MSG_COLOR
      vars:
        - name: ansible_callback_diy_runner_item_on_failed_msg_color
      type: str

    runner_item_on_skipped_msg:
      description: Output to be used for callback runner_item_on_skipped.
      ini:
        - section: callback_diy
          key: runner_item_on_skipped_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_SKIPPED_MSG
      vars:
        - name: ansible_callback_diy_runner_item_on_skipped_msg
      type: str

    runner_item_on_skipped_msg_color:
      description:
        - Output color to be used for O(runner_item_on_skipped_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: runner_item_on_skipped_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_SKIPPED_MSG_COLOR
      vars:
        - name: ansible_callback_diy_runner_item_on_skipped_msg_color
      type: str

    runner_retry_msg:
      description: Output to be used for callback runner_retry.
      ini:
        - section: callback_diy
          key: runner_retry_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_RETRY_MSG
      vars:
        - name: ansible_callback_diy_runner_retry_msg
      type: str

    runner_retry_msg_color:
      description:
        - Output color to be used for O(runner_retry_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: runner_retry_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_RETRY_MSG_COLOR
      vars:
        - name: ansible_callback_diy_runner_retry_msg_color
      type: str

    runner_on_start_msg:
      description: Output to be used for callback runner_on_start.
      ini:
        - section: callback_diy
          key: runner_on_start_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_START_MSG
      vars:
        - name: ansible_callback_diy_runner_on_start_msg
      type: str

    runner_on_start_msg_color:
      description:
        - Output color to be used for O(runner_on_start_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: runner_on_start_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_START_MSG_COLOR
      vars:
        - name: ansible_callback_diy_runner_on_start_msg_color
      type: str

    runner_on_no_hosts_msg:
      description: Output to be used for callback runner_on_no_hosts.
      ini:
        - section: callback_diy
          key: runner_on_no_hosts_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_NO_HOSTS_MSG
      vars:
        - name: ansible_callback_diy_runner_on_no_hosts_msg
      type: str

    runner_on_no_hosts_msg_color:
      description:
        - Output color to be used for O(runner_on_no_hosts_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: runner_on_no_hosts_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_NO_HOSTS_MSG_COLOR
      vars:
        - name: ansible_callback_diy_runner_on_no_hosts_msg_color
      type: str

    playbook_on_setup_msg:
      description: Output to be used for callback playbook_on_setup.
      ini:
        - section: callback_diy
          key: playbook_on_setup_msg
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_SETUP_MSG
      vars:
        - name: ansible_callback_diy_playbook_on_setup_msg
      type: str

    playbook_on_setup_msg_color:
      description:
        - Output color to be used for O(playbook_on_setup_msg).
        - Template should render a L(valid color value,#notes).
      ini:
        - section: callback_diy
          key: playbook_on_setup_msg_color
      env:
        - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_SETUP_MSG_COLOR
      vars:
        - name: ansible_callback_diy_playbook_on_setup_msg_color
      type: str
'''

EXAMPLES = r'''
ansible.cfg: >
  # Enable plugin
  [defaults]
  stdout_callback=community.general.diy

  [callback_diy]
  # Output when playbook starts
  playbook_on_start_msg="DIY output(via ansible.cfg): playbook example: {{ ansible_callback_diy.playbook.file_name }}"
  playbook_on_start_msg_color=yellow

  # Comment out to allow default plugin output
  # playbook_on_play_start_msg="PLAY: starting play {{ ansible_callback_diy.play.name }}"

  # Accept on_skipped_msg or ansible_callback_diy_runner_on_skipped_msg as input vars
  # If neither are supplied, omit the option
  runner_on_skipped_msg="{{ on_skipped_msg | default(ansible_callback_diy_runner_on_skipped_msg) | default(omit) }}"

  # Newline after every callback
  # on_any_msg='{{ " " | join("\n") }}'

playbook.yml: >
  ---
  - name: "Default plugin output: play example"
    hosts: localhost
    gather_facts: false
    tasks:
      - name:  Default plugin output
        ansible.builtin.debug:
          msg: default plugin output

  - name: Override from play vars
    hosts: localhost
    gather_facts: false
    vars:
      ansible_connection: local
      green: "\e[0m\e[38;5;82m"
      yellow: "\e[0m\e[38;5;11m"
      bright_purple: "\e[0m\e[38;5;105m"
      cyan: "\e[0m\e[38;5;51m"
      green_bg_black_fg: "\e[0m\e[48;5;40m\e[38;5;232m"
      yellow_bg_black_fg: "\e[0m\e[48;5;226m\e[38;5;232m"
      purple_bg_white_fg: "\e[0m\e[48;5;57m\e[38;5;255m"
      cyan_bg_black_fg: "\e[0m\e[48;5;87m\e[38;5;232m"
      magenta: "\e[38;5;198m"
      white: "\e[0m\e[38;5;255m"
      ansible_callback_diy_playbook_on_play_start_msg: "\n{{green}}DIY output(via play vars): play example: {{magenta}}{{ansible_callback_diy.play.name}}\n\n"
      ansible_callback_diy_playbook_on_task_start_msg: "DIY output(via play vars): task example: {{ ansible_callback_diy.task.name }}"
      ansible_callback_diy_playbook_on_task_start_msg_color: cyan
      ansible_callback_diy_playbook_on_stats_msg: |+2
                CUSTOM STATS
        ==============================
        {% for key in ansible_callback_diy.stats | sort %}
        {% if ansible_callback_diy.stats[key] %}
        {% if key == 'ok' %}
        {% set color_one = lookup('vars','green_bg_black_fg') %}
        {% set prefix = '      ' %}
        {% set suffix = '     ' %}
        {% set color_two = lookup('vars','green') %}
        {% elif key == 'changed' %}
        {% set color_one = lookup('vars','yellow_bg_black_fg') %}
        {% set prefix = '   ' %}
        {% set suffix = '   ' %}
        {% set color_two = lookup('vars','yellow') %}
        {% elif key == 'processed' %}
        {% set color_one = lookup('vars','purple_bg_white_fg') %}
        {% set prefix = '  ' %}
        {% set suffix = '  ' %}
        {% set color_two = lookup('vars','bright_purple') %}
        {% elif key == 'skipped' %}
        {% set color_one = lookup('vars','cyan_bg_black_fg') %}
        {% set prefix = '   ' %}
        {% set suffix = '   ' %}
        {% set color_two = lookup('vars','cyan') %}
        {% else %}
        {% set color_one = "" %}
        {% set prefix = "" %}
        {% set suffix = "" %}
        {% set color_two = "" %}
        {% endif %}
        {{ color_one }}{{ "%s%s%s" | format(prefix,key,suffix) }}{{ color_two }}: {{ ansible_callback_diy.stats[key] | to_nice_yaml }}
        {% endif %}
        {% endfor %}

    tasks:
      - name: Custom banner with default plugin result output
        ansible.builtin.debug:
          msg: "default plugin output: result example"

      - name: Override from task vars
        ansible.builtin.debug:
          msg: "example {{ two }}"
        changed_when: true
        vars:
          white_fg_red_bg: "\e[0m\e[48;5;1m"
          two: "{{ white_fg_red_bg }}    2    "
          ansible_callback_diy_playbook_on_task_start_msg: "\nDIY output(via task vars): task example: {{ ansible_callback_diy.task.name }}"
          ansible_callback_diy_playbook_on_task_start_msg_color: bright magenta
          ansible_callback_diy_runner_on_ok_msg: "DIY output(via task vars): result example: \n{{ ansible_callback_diy.result.output.msg }}\n"
          ansible_callback_diy_runner_on_ok_msg_color: "{{ 'yellow' if ansible_callback_diy.result.is_changed else 'bright green' }}"

      - name: Suppress output
        ansible.builtin.debug:
          msg: i should not be displayed
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: ""
          ansible_callback_diy_runner_on_ok_msg: ""

      - name: Using alias vars (see ansible.cfg)
        ansible.builtin.debug:
          msg:
        when: false
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: ""
          on_skipped_msg: "DIY output(via task vars): skipped example:\n\e[0m\e[38;5;4m\u25b6\u25b6 {{ ansible_callback_diy.result.task.name }}\n"
          on_skipped_msg_color: white

      - name: Just stdout
        ansible.builtin.command: echo some stdout
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: "\n"
          ansible_callback_diy_runner_on_ok_msg: "{{ ansible_callback_diy.result.output.stdout }}\n"

      - name: Multiline output
        ansible.builtin.debug:
          msg: "{{ multiline }}"
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: "\nDIY output(via task vars): task example: {{ ansible_callback_diy.task.name }}"
          multiline: "line\nline\nline"
          ansible_callback_diy_runner_on_ok_msg: |+2
            some
            {{ ansible_callback_diy.result.output.msg }}
            output

          ansible_callback_diy_playbook_on_task_start_msg_color: bright blue

      - name: Indentation
        ansible.builtin.debug:
          msg: "{{ item.msg }}"
        with_items:
          - { indent: 1, msg: one., color: red }
          - { indent: 2, msg: two.., color: yellow }
          - { indent: 3, msg: three..., color: bright yellow }
        vars:
          ansible_callback_diy_runner_item_on_ok_msg: "{{ ansible_callback_diy.result.output.msg | indent(item.indent, True) }}"
          ansible_callback_diy_runner_item_on_ok_msg_color: "{{ item.color }}"
          ansible_callback_diy_runner_on_ok_msg: "GO!!!"
          ansible_callback_diy_runner_on_ok_msg_color: bright green

      - name: Using lookup and template as file
        ansible.builtin.shell: "echo {% raw %}'output from {{ file_name }}'{% endraw %} > {{ file_name }}"
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: "\nDIY output(via task vars): task example: {{ ansible_callback_diy.task.name }}"
          file_name: diy_file_template_example
          ansible_callback_diy_runner_on_ok_msg: "{{ lookup('template', file_name) }}"

      - name: 'Look at top level vars available to the "runner_on_ok" callback'
        ansible.builtin.debug:
          msg: ''
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: "\nDIY output(via task vars): task example: {{ ansible_callback_diy.task.name }}"
          ansible_callback_diy_runner_on_ok_msg: |+2
            {% for var in (ansible_callback_diy.top_level_var_names|reject('match','vars|ansible_callback_diy.*')) | sort %}
            {{ green }}{{ var }}:
              {{ white }}{{ lookup('vars', var) }}

            {% endfor %}
          ansible_callback_diy_runner_on_ok_msg_color: white

      - name: 'Look at event data available to the "runner_on_ok" callback'
        ansible.builtin.debug:
          msg: ''
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: "\nDIY output(via task vars): task example: {{ ansible_callback_diy.task.name }}"
          ansible_callback_diy_runner_on_ok_msg: |+2
            {% for key in ansible_callback_diy | sort %}
            {{ green }}{{ key }}:
              {{ white }}{{ ansible_callback_diy[key] }}

            {% endfor %}
'''

import sys
from contextlib import contextmanager
from ansible.template import Templar
from ansible.vars.manager import VariableManager
from ansible.plugins.callback.default import CallbackModule as Default
from ansible.module_utils.common.text.converters import to_text


class DummyStdout(object):
    def flush(self):
        pass

    def write(self, b):
        pass

    def writelines(self, l):
        pass


class CallbackModule(Default):
    """
    Callback plugin that allows you to supply your own custom callback templates to be output.
    """
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = 'stdout'
    CALLBACK_NAME = 'community.general.diy'

    DIY_NS = 'ansible_callback_diy'

    @contextmanager
    def _suppress_stdout(self, enabled):
        saved_stdout = sys.stdout
        if enabled:
            sys.stdout = DummyStdout()
        yield
        sys.stdout = saved_stdout

    def _get_output_specification(self, loader, variables):
        _ret = {}
        _calling_method = sys._getframe(1).f_code.co_name
        _callback_type = (_calling_method[3:] if _calling_method[:3] == "v2_" else _calling_method)
        _callback_options = ['msg', 'msg_color']

        for option in _callback_options:
            _option_name = '%s_%s' % (_callback_type, option)
            _option_template = variables.get(
                self.DIY_NS + "_" + _option_name,
                self.get_option(_option_name)
            )
            _ret.update({option: self._template(
                loader=loader,
                template=_option_template,
                variables=variables
            )})

        _ret.update({'vars': variables})

        return _ret

    def _using_diy(self, spec):
        return (spec['msg'] is not None) and (spec['msg'] != spec['vars']['omit'])

    def _parent_has_callback(self):
        return hasattr(super(CallbackModule, self), sys._getframe(1).f_code.co_name)

    def _template(self, loader, template, variables):
        _templar = Templar(loader=loader, variables=variables)
        return _templar.template(
            template,
            preserve_trailing_newlines=True,
            convert_data=False,
            escape_backslashes=True
        )

    def _output(self, spec, stderr=False):
        _msg = to_text(spec['msg'])
        if len(_msg) > 0:
            self._display.display(msg=_msg, color=spec['msg_color'], stderr=stderr)

    def _get_vars(self, playbook, play=None, host=None, task=None, included_file=None,
                  handler=None, result=None, stats=None, remove_attr_ref_loop=True):
        def _get_value(obj, attr=None, method=None):
            if attr:
                return getattr(obj, attr, getattr(obj, "_" + attr, None))

            if method:
                _method = getattr(obj, method)
                return _method()

        def _remove_attr_ref_loop(obj, attributes):
            _loop_var = getattr(obj, 'loop_control', None)
            _loop_var = (_loop_var or 'item')

            for attr in attributes:
                if str(_loop_var) in str(_get_value(obj=obj, attr=attr)):
                    attributes.remove(attr)

            return attributes

        class CallbackDIYDict(dict):
            def __deepcopy__(self, memo):
                return self

        _ret = {}

        _variable_manager = VariableManager(loader=playbook.get_loader())

        _all = _variable_manager.get_vars()
        if play:
            _all = play.get_variable_manager().get_vars(
                play=play,
                host=(host if host else getattr(result, '_host', None)),
                task=(handler if handler else task)
            )
        _ret.update(_all)

        _ret.update(_ret.get(self.DIY_NS, {self.DIY_NS: CallbackDIYDict()}))

        _ret[self.DIY_NS].update({'playbook': {}})
        _playbook_attributes = ['entries', 'file_name', 'basedir']

        for attr in _playbook_attributes:
            _ret[self.DIY_NS]['playbook'].update({attr: _get_value(obj=playbook, attr=attr)})

        if play:
            _ret[self.DIY_NS].update({'play': {}})
            _play_attributes = ['any_errors_fatal', 'become', 'become_flags', 'become_method',
                                'become_user', 'check_mode', 'collections', 'connection',
                                'debugger', 'diff', 'environment', 'fact_path', 'finalized',
                                'force_handlers', 'gather_facts', 'gather_subset',
                                'gather_timeout', 'handlers', 'hosts', 'ignore_errors',
                                'ignore_unreachable', 'included_conditional', 'included_path',
                                'max_fail_percentage', 'module_defaults', 'name', 'no_log',
                                'only_tags', 'order', 'port', 'post_tasks', 'pre_tasks',
                                'remote_user', 'removed_hosts', 'roles', 'run_once', 'serial',
                                'skip_tags', 'squashed', 'strategy', 'tags', 'tasks', 'uuid',
                                'validated', 'vars_files', 'vars_prompt']

            for attr in _play_attributes:
                _ret[self.DIY_NS]['play'].update({attr: _get_value(obj=play, attr=attr)})

        if host:
            _ret[self.DIY_NS].update({'host': {}})
            _host_attributes = ['name', 'uuid', 'address', 'implicit']

            for attr in _host_attributes:
                _ret[self.DIY_NS]['host'].update({attr: _get_value(obj=host, attr=attr)})

        if task:
            _ret[self.DIY_NS].update({'task': {}})
            _task_attributes = ['action', 'any_errors_fatal', 'args', 'async', 'async_val',
                                'become', 'become_flags', 'become_method', 'become_user',
                                'changed_when', 'check_mode', 'collections', 'connection',
                                'debugger', 'delay', 'delegate_facts', 'delegate_to', 'diff',
                                'environment', 'failed_when', 'finalized', 'ignore_errors',
                                'ignore_unreachable', 'loop', 'loop_control', 'loop_with',
                                'module_defaults', 'name', 'no_log', 'notify', 'parent', 'poll',
                                'port', 'register', 'remote_user', 'retries', 'role', 'run_once',
                                'squashed', 'tags', 'untagged', 'until', 'uuid', 'validated',
                                'when']

            # remove arguments that reference a loop var because they cause templating issues in
            # callbacks that do not have the loop context(e.g. playbook_on_task_start)
            if task.loop and remove_attr_ref_loop:
                _task_attributes = _remove_attr_ref_loop(obj=task, attributes=_task_attributes)

            for attr in _task_attributes:
                _ret[self.DIY_NS]['task'].update({attr: _get_value(obj=task, attr=attr)})

        if included_file:
            _ret[self.DIY_NS].update({'included_file': {}})
            _included_file_attributes = ['args', 'filename', 'hosts', 'is_role', 'task']

            for attr in _included_file_attributes:
                _ret[self.DIY_NS]['included_file'].update({attr: _get_value(
                    obj=included_file,
                    attr=attr
                )})

        if handler:
            _ret[self.DIY_NS].update({'handler': {}})
            _handler_attributes = ['action', 'any_errors_fatal', 'args', 'async', 'async_val',
                                   'become', 'become_flags', 'become_method', 'become_user',
                                   'changed_when', 'check_mode', 'collections', 'connection',
                                   'debugger', 'delay', 'delegate_facts', 'delegate_to', 'diff',
                                   'environment', 'failed_when', 'finalized', 'ignore_errors',
                                   'ignore_unreachable', 'listen', 'loop', 'loop_control',
                                   'loop_with', 'module_defaults', 'name', 'no_log',
                                   'notified_hosts', 'notify', 'parent', 'poll', 'port',
                                   'register', 'remote_user', 'retries', 'role', 'run_once',
                                   'squashed', 'tags', 'untagged', 'until', 'uuid', 'validated',
                                   'when']

            if handler.loop and remove_attr_ref_loop:
                _handler_attributes = _remove_attr_ref_loop(obj=handler,
                                                            attributes=_handler_attributes)

            for attr in _handler_attributes:
                _ret[self.DIY_NS]['handler'].update({attr: _get_value(obj=handler, attr=attr)})

            _ret[self.DIY_NS]['handler'].update({'is_host_notified': handler.is_host_notified(host)})

        if result:
            _ret[self.DIY_NS].update({'result': {}})
            _result_attributes = ['host', 'task', 'task_name']

            for attr in _result_attributes:
                _ret[self.DIY_NS]['result'].update({attr: _get_value(obj=result, attr=attr)})

            _result_methods = ['is_changed', 'is_failed', 'is_skipped', 'is_unreachable']

            for method in _result_methods:
                _ret[self.DIY_NS]['result'].update({method: _get_value(obj=result, method=method)})

            _ret[self.DIY_NS]['result'].update({'output': getattr(result, '_result', None)})

            _ret.update(result._result)

        if stats:
            _ret[self.DIY_NS].update({'stats': {}})
            _stats_attributes = ['changed', 'custom', 'dark', 'failures', 'ignored',
                                 'ok', 'processed', 'rescued', 'skipped']

            for attr in _stats_attributes:
                _ret[self.DIY_NS]['stats'].update({attr: _get_value(obj=stats, attr=attr)})

        _ret[self.DIY_NS].update({'top_level_var_names': list(_ret.keys())})

        return _ret

    def v2_on_any(self, *args, **kwargs):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._diy_spec['vars']
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_on_any(*args, **kwargs)

    def v2_runner_on_failed(self, result, ignore_errors=False):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec, stderr=(not ignore_errors))

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_runner_on_failed(result, ignore_errors)

    def v2_runner_on_ok(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_runner_on_ok(result)

    def v2_runner_on_skipped(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_runner_on_skipped(result)

    def v2_runner_on_unreachable(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_runner_on_unreachable(result)

    # not implemented as the call to this is not implemented yet
    def v2_runner_on_async_poll(self, result):
        pass

    # not implemented as the call to this is not implemented yet
    def v2_runner_on_async_ok(self, result):
        pass

    # not implemented as the call to this is not implemented yet
    def v2_runner_on_async_failed(self, result):
        pass

    def v2_runner_item_on_ok(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result,
                remove_attr_ref_loop=False
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_runner_item_on_ok(result)

    def v2_runner_item_on_failed(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result,
                remove_attr_ref_loop=False
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_runner_item_on_failed(result)

    def v2_runner_item_on_skipped(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result,
                remove_attr_ref_loop=False
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_runner_item_on_skipped(result)

    def v2_runner_retry(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_runner_retry(result)

    def v2_runner_on_start(self, host, task):
        self._diy_host = host
        self._diy_task = task

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                host=self._diy_host,
                task=self._diy_task
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_runner_on_start(host, task)

    def v2_playbook_on_start(self, playbook):
        self._diy_playbook = playbook
        self._diy_loader = self._diy_playbook.get_loader()

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_playbook_on_start(playbook)

    def v2_playbook_on_notify(self, handler, host):
        self._diy_handler = handler
        self._diy_host = host

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                host=self._diy_host,
                handler=self._diy_handler
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_playbook_on_notify(handler, host)

    def v2_playbook_on_no_hosts_matched(self):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._diy_spec['vars']
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_playbook_on_no_hosts_matched()

    def v2_playbook_on_no_hosts_remaining(self):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._diy_spec['vars']
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_playbook_on_no_hosts_remaining()

    def v2_playbook_on_task_start(self, task, is_conditional):
        self._diy_task = task

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_playbook_on_task_start(task, is_conditional)

    # not implemented as the call to this is not implemented yet
    def v2_playbook_on_cleanup_task_start(self, task):
        pass

    def v2_playbook_on_handler_task_start(self, task):
        self._diy_task = task

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_playbook_on_handler_task_start(task)

    def v2_playbook_on_vars_prompt(self, varname, private=True, prompt=None, encrypt=None,
                                   confirm=False, salt_size=None, salt=None, default=None,
                                   unsafe=None):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._diy_spec['vars']
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_playbook_on_vars_prompt(
                    varname, private, prompt, encrypt,
                    confirm, salt_size, salt, default,
                    unsafe
                )

    # not implemented as the call to this is not implemented yet
    def v2_playbook_on_import_for_host(self, result, imported_file):
        pass

    # not implemented as the call to this is not implemented yet
    def v2_playbook_on_not_import_for_host(self, result, missing_file):
        pass

    def v2_playbook_on_play_start(self, play):
        self._diy_play = play

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_playbook_on_play_start(play)

    def v2_playbook_on_stats(self, stats):
        self._diy_stats = stats

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                stats=self._diy_stats
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_playbook_on_stats(stats)

    def v2_playbook_on_include(self, included_file):
        self._diy_included_file = included_file

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_included_file._task,
                included_file=self._diy_included_file
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_playbook_on_include(included_file)

    def v2_on_file_diff(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result
            )
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super(CallbackModule, self).v2_on_file_diff(result)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
---
module: aci_l3out_interface_secondary_ip
short_description: Manage Layer 3 Outside (L3Out) interface secondary IP addresses (l3ext:Ip).
description:
- Manage Layer 3 Outside (L3Out) interface secondary IP addresses (l3ext:Ip).
options:
  tenant:
    description:
    - Name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  l3out:
    description:
    - Name of an existing L3Out.
    type: str
    aliases: [ l3out_name ]
  node_profile:
    description:
    - Name of the node profile.
    type: str
    aliases: [ node_profile_name, logical_node ]
  interface_profile:
    description:
    - Name of the interface profile.
    type: str
    aliases: [ interface_profile_name, logical_interface ]
  pod_id:
    description:
    - Pod to build the interface on.
    type: str
  node_id:
    description:
    - Node to build the interface on for Port-channels and single ports.
    - Hyphen separated pair of nodes (e.g. "201-202") for vPCs.
    type: str
  path_ep:
    description:
    - Path to interface
    - Interface Policy Group name for Port-channels and vPCs
    - Port number for single ports (e.g. "eth1/12")
    type: str
  side:
    description:
    - Provides the side for vPC member interfaces.
    type: str
    choices: [ A, B ]
  address:
    description:
    - Secondary IP address.
    type: str
    aliases: [ addr, ip_address]
  ipv6_dad:
    description:
    - IPv6 DAD feature.
    type: str
    choices: [ enabled, disabled]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- This is a test
seealso:
- module: aci_l3out
- module: aci_l3out_logical_node_profile
- module: aci_l3out_logical_interface_profile
- module: aci_l3out_logical_interface
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(l3ext:RsPathL3OutAtt)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Marcel Zehnder (@maercu)
"""

EXAMPLES = r"""
- name: Add a new secondary IP to a routed interface
  cisco.aci.aci_l3out_interface_secondary_ip:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201
    path_ep: eth1/12
    address: 192.168.10.2/27
    state: present
  delegate_to: localhost

- name: Add a new secondary IP to a vPC member
  cisco.aci.aci_l3out_interface_secondary_ip:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201-202
    path_ep: my_vpc_ipg
    side: A
    address: 192.168.10.2/27
    state: present
  delegate_to: localhost

- name: Delete a secondary IP
  cisco.aci.aci_l3out_interface_secondary_ip:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201
    path_ep: eth1/12
    address: 192.168.10.2/27
    state: absent
  delegate_to: localhost

- name: Query a secondary IP
  cisco.aci.aci_l3out_interface_secondary_ip:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201
    path_ep: eth1/12
    address: 192.168.10.2/27
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible.module_utils.basic import AnsibleModule


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        l3out=dict(type="str", aliases=["l3out_name"]),
        node_profile=dict(type="str", aliases=["node_profile_name", "logical_node"]),
        interface_profile=dict(type="str", aliases=["interface_profile_name", "logical_interface"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        pod_id=dict(type="str"),
        node_id=dict(type="str"),
        path_ep=dict(type="str"),
        side=dict(type="str", choices=["A", "B"]),
        address=dict(type="str", aliases=["addr", "ip_address"]),
        ipv6_dad=dict(type="str", choices=["enabled", "disabled"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            [
                "state",
                "absent",
                [
                    "tenant",
                    "l3out",
                    "node_profile",
                    "interface_profile",
                    "pod_id",
                    "node_id",
                    "path_ep",
                ],
            ],
            [
                "state",
                "present",
                [
                    "tenant",
                    "l3out",
                    "node_profile",
                    "interface_profile",
                    "pod_id",
                    "node_id",
                    "path_ep",
                ],
            ],
        ],
    )

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    node_profile = module.params.get("node_profile")
    interface_profile = module.params.get("interface_profile")
    pod_id = module.params.get("pod_id")
    node_id = module.params.get("node_id")
    path_ep = module.params.get("path_ep")
    side = module.params.get("side")
    address = module.params.get("address")
    ipv6_dad = module.params.get("ipv6_dad")
    state = module.params.get("state")

    aci = ACIModule(module)

    path_type = "paths"
    rn_prefix = ""

    if node_id:
        if "-" in node_id:
            path_type = "protpaths"
            rn_prefix = "mem-{0}/".format(side)

    path_dn = None
    if pod_id and node_id and path_ep:
        path_dn = "topology/pod-{0}/{1}-{2}/pathep-[{3}]".format(pod_id, path_type, node_id, path_ep)

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extLNodeP",
            aci_rn="lnodep-{0}".format(node_profile),
            module_object=node_profile,
            target_filter={"name": node_profile},
        ),
        subclass_3=dict(
            aci_class="l3extLIfP",
            aci_rn="lifp-{0}".format(interface_profile),
            module_object=interface_profile,
            target_filter={"name": interface_profile},
        ),
        subclass_4=dict(
            aci_class="l3extRsPathL3OutAtt",
            aci_rn="rspathL3OutAtt-[{0}]".format(path_dn),
            module_object=path_dn,
            target_filter={"tDn": path_dn},
        ),
        subclass_5=dict(
            aci_class="l3extIp",
            aci_rn="{0}addr-[{1}]".format(rn_prefix, address),
            module_object=address,
            target_filter={"addr": address},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(aci_class="l3extIp", class_config=dict(addr=address, ipv6Dad=ipv6_dad))

        aci.get_diff(aci_class="l3extIp")
        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2017-2020 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):

    # Standard files documentation fragment
    DOCUMENTATION = r'''
requirements:
    - lxml

options:
    hosttech_username:
        description:
          - The username for the Hosttech API user.
          - If provided, O(hosttech_password) must also be provided.
          - Mutually exclusive with O(hosttech_token).
        type: str
    hosttech_password:
        description:
          - The password for the Hosttech API user.
          - If provided, O(hosttech_username) must also be provided.
          - Mutually exclusive with O(hosttech_token).
        type: str
    hosttech_token:
        description:
          - The password for the Hosttech API user.
          - Mutually exclusive with O(hosttech_username) and O(hosttech_password).
          - Since community.dns 1.2.0, the alias O(ignore:api_token) can be used.
        aliases:
          - api_token
        type: str
        version_added: 0.2.0
'''

    # NOTE: This document fragment augments the above standard DOCUMENTATION document fragment
    #       by providing alternative ways to provide configuration for plugins. (The above
    #       documentation fragment is tailored for modules.)
    PLUGIN = r'''
options:
    hosttech_username:
        env:
          - name: ANSIBLE_HOSTTECH_API_USERNAME
            version_added: 2.5.0
    hosttech_password:
        env:
          - name: ANSIBLE_HOSTTECH_API_PASSWORD
            version_added: 2.5.0
    hosttech_token:
        env:
          - name: ANSIBLE_HOSTTECH_DNS_TOKEN
            version_added: 2.5.0
'''

    # NOTE: This document fragment adds additional information on records.
    RECORD_NOTES = r'''
options: {}
'''

    # WARNING: This section is automatically generated by update-docs-fragments.py.
    #          It is used to augment the docs fragments module_record, module_record_set.
    #          DO NOT EDIT MANUALLY!
    RECORD_DEFAULT_TTL = r'''
options:
    ttl:
        default: 3600
'''

    # WARNING: This section is automatically generated by update-docs-fragments.py.
    #          It is used to augment the docs fragments module_record, module_record_info,
    #          module_record_set, module_record_set_info.
    #          DO NOT EDIT MANUALLY!
    RECORD_TYPE_CHOICES = r'''
options:
    type:
        choices:
          - A
          - AAAA
          - CAA
          - CNAME
          - MX
          - NS
          - PTR
          - SPF
          - SRV
          - TXT
'''

    # WARNING: This section is automatically generated by update-docs-fragments.py.
    #          It is used to augment the docs fragment module_record_sets.
    #          DO NOT EDIT MANUALLY!
    RECORD_TYPE_CHOICES_RECORD_SETS_MODULE = r'''
options:
    record_sets:
        suboptions:
            record:
                description:
                  - The full DNS record to create or delete.
                  - Exactly one of O(record_sets[].record) and O(record_sets[].prefix)
                      must be specified.
                type: str
            prefix:
                description:
                  - The prefix of the DNS record.
                  - This is the part of O(record_sets[].record) before O(zone_name).
                      For example, if the record to be modified is C(www.example.com)
                      for the zone C(example.com), the prefix is V(www). If the record
                      in this example would be C(example.com), the prefix would be
                      V('') (empty string).
                  - Exactly one of O(record_sets[].record) and O(record_sets[].prefix)
                      must be specified.
                type: str
            ttl:
                description:
                  - The TTL to give the new record, in seconds.
                type: int
                default: 3600
            type:
                description:
                  - The type of DNS record to create or delete.
                required: true
                type: str
                choices:
                  - A
                  - AAAA
                  - CAA
                  - CNAME
                  - MX
                  - NS
                  - PTR
                  - SPF
                  - SRV
                  - TXT
            value:
                description:
                  - The new value when creating a DNS record.
                  - YAML lists or multiple comma-spaced values are allowed.
                  - When deleting a record all values for the record must be specified
                      or it will not be deleted.
                  - Must be specified if O(record_sets[].ignore=false).
                type: list
                elements: str
            ignore:
                description:
                  - If set to V(true), O(record_sets[].value) will be ignored.
                  - This is useful when O(prune=true), but you do not want certain
                      entries to be removed without having to know their current value.
                type: bool
                default: false
'''

    # WARNING: This section is automatically generated by update-docs-fragments.py.
    #          It is used to augment the docs fragment inventory_records.
    #          DO NOT EDIT MANUALLY!
    RECORD_TYPE_CHOICES_RECORDS_INVENTORY = r'''
options:
    simple_filters:
        suboptions:
            type:
                description:
                  - Record types whose values to use.
                type: list
                elements: string
                default:
                  - A
                  - AAAA
                  - CNAME
                choices:
                  - A
                  - AAAA
                  - CAA
                  - CNAME
                  - MX
                  - NS
                  - PTR
                  - SPF
                  - SRV
                  - TXT
'''

    # WARNING: This section is automatically generated by update-docs-fragments.py.
    #          It is used to augment the docs fragments inventory_records, module_record,
    #          module_record_info, module_record_set, module_record_set_info,
    #          module_record_sets, module_zone_info.
    #          DO NOT EDIT MANUALLY!
    ZONE_ID_TYPE = r'''
options:
    zone_id:
        type: int
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_set_domain
short_description: Edit existing object using object name or uid.
description:
  - Edit existing object using object name or uid.
  - All operations are performed over Web Services API.
version_added: "2.1.0"
author: "Or Soffer (@chkp-orso)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  servers:
    description:
      - Domain servers. When this field is provided, 'set-domain' command is executed asynchronously.
    type: dict
    suboptions:
      add:
        description:
          - Adds to collection of values
        type: list
        elements: dict
        suboptions:
          name:
            description:
              - Object name. Must be unique in the domain.
            type: str
          ip_address:
            description:
              - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly.
            type: str
          ipv4_address:
            description:
              - IPv4 address.
            type: str
          ipv6_address:
            description:
              - IPv6 address.
            type: str
          multi_domain_server:
            description:
              - Multi Domain server name or UID.
            type: str
          skip_start_domain_server:
            description:
              - Set this value to be true to prevent starting the new created domain.
            type: bool
          type:
            description:
              - Domain server type.
            type: str
            choices: ['management server', 'log server', 'smc']
      remove:
        description:
          - Remove from collection of values
        type: list
        elements: str
        suboptions:
          name:
            description:
               - Object name. Must be unique in the domain.
            type: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
  tags:
    description:
      - Collection of tag identifiers. Note, The list of tags can not be modified in a single command together with the domain servers. To modify
        tags, please use the separate 'set-domain' command, without providing the list of domain servers.
    type: list
    elements: str
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: set-domain
  cp_mgmt_set_domain:
    comments: This is domain1 comment
    name: domain1
"""

RETURN = """
cp_mgmt_domain:
  description: The checkpoint set-domain output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        servers=dict(
            type="dict",
            options=dict(
                add=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        name=dict(type="str"),
                        ip_address=dict(type="str"),
                        ipv4_address=dict(type="str"),
                        ipv6_address=dict(type="str"),
                        multi_domain_server=dict(type="str"),
                        skip_start_domain_server=dict(type="bool"),
                        type=dict(
                            type="str",
                            choices=["management server", "log server", "smc"],
                        ),
                    ),
                ),
                remove=dict(type="list", elements="str"),
            ),
        ),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
        tags=dict(type="list", elements="str"),
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)
    command = "set-domain"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################
"""
The module file for nxos_lldp_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_lldp_interfaces
short_description: LLDP interfaces resource module
description: This module manages interfaces' configuration for Link Layer Discovery
  Protocol (LLDP) on NX-OS platforms.
version_added: 1.0.0
author: Adharsh Srivats Rangarajan (@adharshsrivatsr)
notes:
- Tested against NXOS 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
- The LLDP feature needs to be enabled before using this module
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the command B(show running-config | section ^interface).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description:
    - A list of link layer discovery configurations for interfaces.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Name of the interface
        required: true
        type: str
      receive:
        description:
        - Used to enable or disable the reception of LLDP packets on that interface.
          By default, this is enabled after LLDP is enabled globally.
        type: bool
      transmit:
        description:
        - Used to enable or disable the transmission of LLDP packets on that interface.
          By default, this is enabled after LLDP is enabled globally.
        type: bool
      tlv_set:
        description:
        - Used to configure TLV parameters on the interface
        type: dict
        suboptions:
          management_address:
            description:
            - Used to mention the IPv4 or IPv6 management address for the interface
            type: str
          vlan:
            description:
            - Used to mention the VLAN for the interface
            type: int
  state:
    description:
    - The state the configuration should be left in
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - gathered
    - rendered
    - parsed
    default: merged

"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
#

- name: Merge provided configuration with device configuration
  cisco.nxos.nxos_lldp_interfaces:
    config:
    - name: Ethernet1/4
      receive: false
      transmit: true
      tlv_set:
        management_address: 192.168.122.64
      vlan: 12
    state: merged

# After state:
# -------------
#
# interface Ethernet1/4
#   no lldp receive
#   lldp tlv-set management-address 192.168.122.64
#   lldp tlv-set vlan 12


# Using replaced

# Before state:
# ------------
#
# interface Ethernet1/4
#   no lldp receive
#   lldp tlv-set management-address 192.168.122.64
# interface Ethernet1/5
#   no lldp transmit
#   lldp tlv-set vlan 10

- name: Replace LLDP configuration on interfaces with given configuration
  cisco.nxos.nxos_lldp_interfaces:
    config:
    - name: Ethernet1/4
      transmit: false
      tlv_set:
        vlan: 2
    state: replaced


# After state:
# -----------
#
# interface Ethernet1/4
#   no lldp transmit
#   lldp tlv_set vlan 2
# interface Ethernet1/5
#   no lldp transmit
#   lldp tlv-set vlan 10


# Using overridden

# Before state:
# ------------
#
# interface Ethernet1/4
#   no lldp receive
#   lldp tlv-set management-address 192.168.122.64
# interface Ethernet1/5
#   no lldp transmit
#   lldp tlv-set vlan 10

- name: Override LLDP configuration on all interfaces with given configuration
  cisco.nxos.nxos_lldp_interfaces:
    config:
    - name: Ethernet1/7
      receive: false
      tlv_set:
        vlan: 12
    state: overridden


# After state:
# -----------
#
# interface Ethernet1/7
#   no lldp receive
#   lldp tlv_set vlan 12


# Using deleted

# Before state:
# ------------
#
# interface Ethernet1/4
#   lldp tlv-set management vlan 24
#   no lldp transmit
# interface mgmt0
#   no lldp receive

- name: Delete LLDP interfaces configuration
  cisco.nxos.nxos_lldp_interfaces:
    state: deleted

# After state:
# ------------
#


"""
RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['interface Ethernet1/2', 'lldp receive', 'lldp tlv-set vlan 12']
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.lldp_interfaces.lldp_interfaces import (
    Lldp_interfacesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.lldp_interfaces.lldp_interfaces import (
    Lldp_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Lldp_interfacesArgs.argument_spec,
        supports_check_mode=True,
    )

    result = Lldp_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: aci_settings
short_description: Resource module for ACI Settings
description:
- Manage operation update of the resource ACI Settings.
- This API allows the client to update ACI settings.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  aci50:
    description: Enable 5.0 ACI Version.
    type: bool
  aci51:
    description: Enable 5.1 ACI Version.
    type: bool
  aciipaddress:
    description: ACI Domain manager Ip Address.
    type: str
  acipassword:
    description: ACI Domain manager Password.
    type: str
  aciuserName:
    description: ACI Domain manager Username.
    type: str
  adminName:
    description: ACI Cluster Admin name.
    type: str
  adminPassword:
    description: ACI Cluster Admin password.
    type: str
  allSXPDomain:
    description: AllSXPDomain flag.
    type: bool
  defaultSGtName:
    description: ACI Settings's defaultSGtName.
    type: str
  enableACI:
    description: Enable ACI Integration.
    type: bool
  enableDataPlane:
    description: EnableDataPlane flag.
    type: bool
  enableElementsLimit:
    description: EnableElementsLimit flag.
    type: bool
  id:
    description: Resource UUID value.
    type: str
  ipAddressHostName:
    description: ACI Cluster IP Address / Host name.
    type: str
  l3RouteNetwork:
    description: ACI Settings's l3RouteNetwork.
    type: str
  maxNumIepgFromACI:
    description: ACI Settings's maxNumIepgFromACI.
    type: int
  maxNumSGtToACI:
    description: ACI Settings's maxNumSGtToACI.
    type: int
  specificSXPDomain:
    description: SpecificSXPDomain flag.
    type: bool
  specifixSXPDomainList:
    description: ACI Settings's specifixSXPDomainList.
    elements: str
    type: list
  suffixToEpg:
    description: ACI Settings's suffixToEpg.
    type: str
  suffixToSGt:
    description: ACI Settings's suffixToSGt.
    type: str
  tenantName:
    description: ACI Settings's tenantName.
    type: str
  untaggedPacketIepgName:
    description: ACI Settings's untaggedPacketIepgName.
    type: str
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
notes:
  - SDK Method used are
    aci_settings.AciSettings.update_aci_settings_by_id,

  - Paths used are
    put /ers/config/acisettings/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.aci_settings:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    aci50: true
    aci51: true
    aciipaddress: string
    acipassword: string
    aciuserName: string
    adminName: string
    adminPassword: string
    allSxpDomain: true
    defaultSgtName: string
    enableAci: true
    enableDataPlane: true
    enableElementsLimit: true
    id: string
    ipAddressHostName: string
    l3RouteNetwork: string
    maxNumIepgFromAci: 0
    maxNumSgtToAci: 0
    specificSxpDomain: true
    specifixSxpDomainList:
    - string
    suffixToEpg: string
    suffixToSgt: string
    tenantName: string
    untaggedPacketIepgName: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "enableAci": true,
      "ipAddressHostName": "string",
      "adminName": "string",
      "adminPassword": "string",
      "aciipaddress": "string",
      "aciuserName": "string",
      "acipassword": "string",
      "tenantName": "string",
      "l3RouteNetwork": "string",
      "suffixToEpg": "string",
      "suffixToSgt": "string",
      "allSxpDomain": true,
      "specificSxpDomain": true,
      "specifixSxpDomainList": [
        "string"
      ],
      "enableDataPlane": true,
      "untaggedPacketIepgName": "string",
      "defaultSgtName": "string",
      "enableElementsLimit": true,
      "maxNumIepgFromAci": 0,
      "maxNumSgtToAci": 0,
      "aci50": true,
      "aci51": true
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_clients_splash_authorization_status
short_description: Resource module for networks _clients _splashauthorizationstatus
description:
- Manage operation update of the resource networks _clients _splashauthorizationstatus.
- >
   Update a client's splash authorization. Clients can be identified by a client key or either the MAC or IP
   depending on whether the network uses Track-by-IP.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  clientId:
    description: ClientId path parameter. Client ID.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  ssids:
    description: The target SSIDs. Each SSID must be enabled and must have Click-through
      splash enabled. For each SSID where isAuthorized is true, the expiration time
      will automatically be set according to the SSID's splash frequency. Not all networks
      support configuring all SSIDs.
    suboptions:
      '0':
        description: Splash authorization for SSID 0.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '1':
        description: Splash authorization for SSID 1.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '10':
        description: Splash authorization for SSID 10.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '11':
        description: Splash authorization for SSID 11.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '12':
        description: Splash authorization for SSID 12.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '13':
        description: Splash authorization for SSID 13.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '14':
        description: Splash authorization for SSID 14.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '2':
        description: Splash authorization for SSID 2.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '3':
        description: Splash authorization for SSID 3.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '4':
        description: Splash authorization for SSID 4.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '5':
        description: Splash authorization for SSID 5.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '6':
        description: Splash authorization for SSID 6.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '7':
        description: Splash authorization for SSID 7.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '8':
        description: Splash authorization for SSID 8.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
      '9':
        description: Splash authorization for SSID 9.
        suboptions:
          isAuthorized:
            description: New authorization status for the SSID (true, false).
            type: bool
        type: dict
    type: dict
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for networks updateNetworkClientSplashAuthorizationStatus
  description: Complete reference of the updateNetworkClientSplashAuthorizationStatus API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-client-splash-authorization-status
notes:
  - SDK Method used are
    networks.Networks.update_network_client_splash_authorization_status,

  - Paths used are
    put /networks/{networkId}/clients/{clientId}/splashAuthorizationStatus,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_clients_splash_authorization_status:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    clientId: string
    networkId: string
    ssids:
      '0':
        isAuthorized: true
      '2':
        isAuthorized: false

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_appliance_static_routes
short_description: Resource module for networks _appliance _static _routes
description:
- Manage operations create, update and delete of the resource networks _appliance _static _routes.
- Add a static route for an MX or teleworker network.
- Delete a static route from an MX or teleworker network.
- Update a static route for an MX or teleworker network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  enabled:
    description: The enabled state of the static route.
    type: bool
  fixedIpAssignments:
    description: The DHCP fixed IP assignments on the static route. This should be an
      object that contains mappings from MAC addresses to objects that themselves each
      contain "ip" and "name" string fields. See the sample request/response for more
      details.
    type: dict
  gatewayIp:
    description: The gateway IP (next hop) of the static route.
    type: str
  gatewayVlanId:
    description: The gateway IP (next hop) VLAN ID of the static route.
    type: str
  name:
    description: The name of the new static route.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  reservedIpRanges:
    description: The DHCP reserved IP ranges on the static route.
    elements: dict
    suboptions:
      comment:
        description: A text comment for the reserved range.
        type: str
      end:
        description: The last IP in the reserved range.
        type: str
      start:
        description: The first IP in the reserved range.
        type: str
    type: list
  staticRouteId:
    description: StaticRouteId path parameter. Static route ID.
    type: str
  subnet:
    description: The subnet of the static route.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for appliance createNetworkApplianceStaticRoute
  description: Complete reference of the createNetworkApplianceStaticRoute API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-network-appliance-static-route
- name: Cisco Meraki documentation for appliance deleteNetworkApplianceStaticRoute
  description: Complete reference of the deleteNetworkApplianceStaticRoute API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-network-appliance-static-route
- name: Cisco Meraki documentation for appliance updateNetworkApplianceStaticRoute
  description: Complete reference of the updateNetworkApplianceStaticRoute API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-static-route
notes:
  - SDK Method used are
    appliance.Appliance.create_network_appliance_static_route,
    appliance.Appliance.delete_network_appliance_static_route,
    appliance.Appliance.update_network_appliance_static_route,

  - Paths used are
    post /networks/{networkId}/appliance/staticRoutes,
    delete /networks/{networkId}/appliance/staticRoutes/{staticRouteId},
    put /networks/{networkId}/appliance/staticRoutes/{staticRouteId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_appliance_static_routes:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_caller: "{{meraki_caller}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    gatewayIp: 1.2.3.5
    name: My route
    networkId: string
    subnet: 192.168.1.0/24

- name: Update by id
  cisco.meraki.networks_appliance_static_routes:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_caller: "{{meraki_caller}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    fixedIpAssignments:
      22:33:44:55:66:77:
        ip: 1.2.3.4
        name: Some client name
    name: My route
    networkId: string
    reservedIpRanges:
    - comment: A reserved IP range
      end: 192.168.1.1
      start: 192.168.1.0
    staticRouteId: string
    subnet: 192.168.1.0/24

- name: Delete by id
  cisco.meraki.networks_appliance_static_routes:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_caller: "{{meraki_caller}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    networkId: string
    staticRouteId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c), Entrust Datacard Corporation, 2019
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: ecs_certificate
author:
    - Chris Trufan (@ctrufan)
short_description: Request SSL/TLS certificates with the Entrust Certificate Services (ECS) API
description:
    - Create, reissue, and renew certificates with the Entrust Certificate Services (ECS) API.
    - Requires credentials for the L(Entrust Certificate Services,https://www.entrustdatacard.com/products/categories/ssl-certificates) (ECS) API.
    - In order to request a certificate, the domain and organization used in the certificate signing request must be already
      validated in the ECS system. It is I(not) the responsibility of this module to perform those steps.
notes:
    - O(path) must be specified as the output location of the certificate.
requirements:
    - cryptography >= 1.6
extends_documentation_fragment:
    - community.crypto.attributes
    - community.crypto.attributes.files
    - community.crypto.ecs_credential
attributes:
    check_mode:
        support: partial
        details:
            - Check mode is only supported if O(request_type=new).
    diff_mode:
        support: none
    safe_file_operations:
        support: full
options:
    backup:
        description:
            - Whether a backup should be made for the certificate in O(path).
        type: bool
        default: false
    force:
        description:
            - If force is used, a certificate is requested regardless of whether O(path) points to an existing valid certificate.
            - If O(request_type=renew), a forced renew will fail if the certificate being renewed has been issued within the past 30 days, regardless of the
              value of O(remaining_days) or the return value of RV(cert_days) - the ECS API does not support the "renew" operation for certificates that are not
              at least 30 days old.
        type: bool
        default: false
    path:
        description:
            - The destination path for the generated certificate as a PEM encoded cert.
            - If the certificate at this location is not an Entrust issued certificate, a new certificate will always be requested even if the current
              certificate is technically valid.
            - If there is already an Entrust certificate at this location, whether it is replaced is depends on the O(remaining_days) calculation.
            - If an existing certificate is being replaced (see O(remaining_days), O(force), and O(tracking_id)), whether a new certificate is requested
              or the existing certificate is renewed or reissued is based on O(request_type).
        type: path
        required: true
    full_chain_path:
        description:
            - The destination path for the full certificate chain of the certificate, intermediates, and roots.
        type: path
    csr:
        description:
            - Base-64 encoded Certificate Signing Request (CSR). O(csr) is accepted with or without PEM formatting around the Base-64 string.
            - If no O(csr) is provided when O(request_type=reissue) or O(request_type=renew), the certificate will be generated with the same public key as
              the certificate being renewed or reissued.
            - If O(subject_alt_name) is specified, it will override the subject alternate names in the CSR.
            - If O(eku) is specified, it will override the extended key usage in the CSR.
            - If O(ou) is specified, it will override the organizational units "ou=" present in the subject distinguished name of the CSR, if any.
            - The organization "O=" field from the CSR will not be used. It will be replaced in the issued certificate by O(org) if present, and if not present,
              the organization tied to O(client_id).
        type: str
    tracking_id:
        description:
            - The tracking ID of the certificate to reissue or renew.
            - O(tracking_id) is invalid if O(request_type=new) or O(request_type=validate_only).
            - If there is a certificate present in O(path) and it is an ECS certificate, O(tracking_id) will be ignored.
            - If there is no certificate present in O(path) or there is but it is from another provider, the certificate represented by O(tracking_id) will
              be renewed or reissued and saved to O(path).
            - If there is no certificate present in O(path) and the O(force) and O(remaining_days) parameters do not indicate a new certificate is needed,
              the certificate referenced by O(tracking_id) certificate will be saved to O(path).
            - This can be used when a known certificate is not currently present on a server, but you want to renew or reissue it to be managed by an ansible
              playbook. For example, if you specify O(request_type=renew), O(tracking_id) of an issued certificate, and O(path) to a file that does not exist,
              the first run of a task will download the certificate specified by O(tracking_id) (assuming it is still valid). Future runs of the task will
              (if applicable - see O(force) and O(remaining_days)) renew the certificate now present in O(path).
        type: int
    remaining_days:
        description:
            - The number of days the certificate must have left being valid. If RV(cert_days) < O(remaining_days) then a new certificate will be
              obtained using O(request_type).
            - If O(request_type=renew), a renewal will fail if the certificate being renewed has been issued within the past 30 days, so do not set a
              O(remaining_days) value that is within 30 days of the full lifetime of the certificate being acted upon.
            - For example, if you are requesting Certificates with a 90 day lifetime, do not set O(remaining_days) to a value V(60) or higher).
            - The O(force) option may be used to ensure that a new certificate is always obtained.
        type: int
        default: 30
    request_type:
        description:
            - The operation performed if O(tracking_id) references a valid certificate to reissue, or there is already a certificate present in O(path) but
              either O(force) is specified or RV(cert_days) < O(remaining_days).
            - Specifying O(request_type=validate_only) means the request will be validated against the ECS API, but no certificate will be issued.
            - Specifying O(request_type=new) means a certificate request will always be submitted and a new certificate issued.
            - Specifying O(request_type=renew) means that an existing certificate (specified by O(tracking_id) if present, otherwise O(path)) will be renewed.
              If there is no certificate to renew, a new certificate is requested.
            - Specifying O(request_type=reissue) means that an existing certificate (specified by O(tracking_id) if present, otherwise O(path)) will be
              reissued.
              If there is no certificate to reissue, a new certificate is requested.
            - If a certificate was issued within the past 30 days, the V(renew) operation is not a valid operation and will fail.
            - Note that V(reissue) is an operation that will result in the revocation of the certificate that is reissued, be cautious with its use.
            - I(check_mode) is only supported if O(request_type=new)
            - For example, setting O(request_type=renew) and O(remaining_days=30) and pointing to the same certificate on multiple playbook runs means that on
              the first run new certificate will be requested. It will then be left along on future runs until it is within 30 days of expiry, then the
              ECS "renew" operation will be performed.
        type: str
        choices: [ 'new', 'renew', 'reissue', 'validate_only']
        default: new
    cert_type:
        description:
            - Specify the type of certificate requested.
            - If a certificate is being reissued or renewed, this parameter is ignored, and the O(cert_type) of the initial certificate is used.
        type: str
        choices: [ 'STANDARD_SSL', 'ADVANTAGE_SSL', 'UC_SSL', 'EV_SSL', 'WILDCARD_SSL', 'PRIVATE_SSL', 'PD_SSL', 'CODE_SIGNING', 'EV_CODE_SIGNING',
                   'CDS_INDIVIDUAL', 'CDS_GROUP', 'CDS_ENT_LITE', 'CDS_ENT_PRO', 'SMIME_ENT' ]
    subject_alt_name:
        description:
            - The subject alternative name identifiers, as an array of values (applies to O(cert_type) with a value of V(STANDARD_SSL), V(ADVANTAGE_SSL),
              V(UC_SSL), V(EV_SSL), V(WILDCARD_SSL), V(PRIVATE_SSL), and V(PD_SSL)).
            - If you are requesting a new SSL certificate, and you pass a O(subject_alt_name) parameter, any SAN names in the CSR are ignored.
              If no subjectAltName parameter is passed, the SAN names in the CSR are used.
            - See O(request_type) to understand more about SANs during reissues and renewals.
            - In the case of certificates of type V(STANDARD_SSL) certificates, if the CN of the certificate is <domain>.<tld> only the www.<domain>.<tld> value
              is accepted. If the CN of the certificate is www.<domain>.<tld> only the <domain>.<tld> value is accepted.
        type: list
        elements: str
    eku:
        description:
            - If specified, overrides the key usage in the O(csr).
        type: str
        choices: [ SERVER_AUTH, CLIENT_AUTH, SERVER_AND_CLIENT_AUTH ]
    ct_log:
        description:
            - In compliance with browser requirements, this certificate may be posted to the Certificate Transparency (CT) logs. This is a best practice
              technique that helps domain owners monitor certificates issued to their domains. Note that not all certificates are eligible for CT logging.
            - If O(ct_log) is not specified, the certificate uses the account default.
            - If O(ct_log) is specified and the account settings allow it, O(ct_log) overrides the account default.
            - If O(ct_log) is set to V(false), but the account settings are set to "always log", the certificate generation will fail.
        type: bool
    client_id:
        description:
            - The client ID to submit the Certificate Signing Request under.
            - If no client ID is specified, the certificate will be submitted under the primary client with ID of 1.
            - When using a client other than the primary client, the O(org) parameter cannot be specified.
            - The issued certificate will have an organization value in the subject distinguished name represented by the client.
        type: int
        default: 1
    org:
        description:
            - Organization "O=" to include in the certificate.
            - If O(org) is not specified, the organization from the client represented by O(client_id) is used.
            - Unless the O(cert_type) is V(PD_SSL), this field may not be specified if the value of O(client_id) is not "1" (the primary client).
              non-primary clients, certificates may only be issued with the organization of that client.
        type: str
    ou:
        description:
            - Organizational unit "OU=" to include in the certificate.
            - O(ou) behavior is dependent on whether organizational units are enabled for your account. If organizational unit support is disabled for your
              account, organizational units from the O(csr) and the O(ou) parameter are ignored.
            - If both O(csr) and O(ou) are specified, the value in O(ou) will override the OU fields present in the subject distinguished name in the O(csr)
            - If neither O(csr) nor O(ou) are specified for a renew or reissue operation, the OU fields in the initial certificate are reused.
            - An invalid OU from O(csr) is ignored, but any invalid organizational units in O(ou) will result in an error indicating "Unapproved OU". The O(ou)
              parameter can be used to force failure if an unapproved organizational unit is provided.
            - A maximum of one OU may be specified for current products. Multiple OUs are reserved for future products.
        type: list
        elements: str
    end_user_key_storage_agreement:
        description:
            - The end user of the Code Signing certificate must generate and store the private key for this request on cryptographically secure
              hardware to be compliant with the Entrust CSP and Subscription agreement. If requesting a certificate of type V(CODE_SIGNING) or
              V(EV_CODE_SIGNING), you must set O(end_user_key_storage_agreement) to true if and only if you acknowledge that you will inform the user of this
              requirement.
            - Applicable only to O(cert_type) of values V(CODE_SIGNING) and V(EV_CODE_SIGNING).
        type: bool
    tracking_info:
        description: Free form tracking information to attach to the record for the certificate.
        type: str
    requester_name:
        description: The requester name to associate with certificate tracking information.
        type: str
        required: true
    requester_email:
        description: The requester email to associate with certificate tracking information and receive delivery and expiry notices for the certificate.
        type: str
        required: true
    requester_phone:
        description: The requester phone number to associate with certificate tracking information.
        type: str
        required: true
    additional_emails:
        description: A list of additional email addresses to receive the delivery notice and expiry notification for the certificate.
        type: list
        elements: str
    custom_fields:
        description:
            - Mapping of custom fields to associate with the certificate request and certificate.
            - Only supported if custom fields are enabled for your account.
            - Each custom field specified must be a custom field you have defined for your account.
        type: dict
        suboptions:
            text1:
                description: Custom text field (maximum 500 characters)
                type: str
            text2:
                description: Custom text field (maximum 500 characters)
                type: str
            text3:
                description: Custom text field (maximum 500 characters)
                type: str
            text4:
                description: Custom text field (maximum 500 characters)
                type: str
            text5:
                description: Custom text field (maximum 500 characters)
                type: str
            text6:
                description: Custom text field (maximum 500 characters)
                type: str
            text7:
                description: Custom text field (maximum 500 characters)
                type: str
            text8:
                description: Custom text field (maximum 500 characters)
                type: str
            text9:
                description: Custom text field (maximum 500 characters)
                type: str
            text10:
                description: Custom text field (maximum 500 characters)
                type: str
            text11:
                description: Custom text field (maximum 500 characters)
                type: str
            text12:
                description: Custom text field (maximum 500 characters)
                type: str
            text13:
                description: Custom text field (maximum 500 characters)
                type: str
            text14:
                description: Custom text field (maximum 500 characters)
                type: str
            text15:
                description: Custom text field (maximum 500 characters)
                type: str
            number1:
                description: Custom number field.
                type: float
            number2:
                description: Custom number field.
                type: float
            number3:
                description: Custom number field.
                type: float
            number4:
                description: Custom number field.
                type: float
            number5:
                description: Custom number field.
                type: float
            date1:
                description: Custom date field.
                type: str
            date2:
                description: Custom date field.
                type: str
            date3:
                description: Custom date field.
                type: str
            date4:
                description: Custom date field.
                type: str
            date5:
                description: Custom date field.
                type: str
            email1:
                description: Custom email field.
                type: str
            email2:
                description: Custom email field.
                type: str
            email3:
                description: Custom email field.
                type: str
            email4:
                description: Custom email field.
                type: str
            email5:
                description: Custom email field.
                type: str
            dropdown1:
                description: Custom dropdown field.
                type: str
            dropdown2:
                description: Custom dropdown field.
                type: str
            dropdown3:
                description: Custom dropdown field.
                type: str
            dropdown4:
                description: Custom dropdown field.
                type: str
            dropdown5:
                description: Custom dropdown field.
                type: str
    cert_expiry:
        description:
            - The date the certificate should be set to expire, in RFC3339 compliant date or date-time format. For example,
              V(2020-02-23), V(2020-02-23T15:00:00.05Z).
            - O(cert_expiry) is only supported for requests of O(request_type=new) or O(request_type=renew). If O(request_type=reissue),
              O(cert_expiry) will be used for the first certificate issuance, but subsequent issuances will have the same expiry as the initial
              certificate.
            - A reissued certificate will always have the same expiry as the original certificate.
            - Note that only the date (day, month, year) is supported for specifying the expiry date. If you choose to specify an expiry time with the expiry
              date, the time will be adjusted to Eastern Standard Time (EST). This could have the unintended effect of moving your expiry date to the previous
              day.
            - Applies only to accounts with a pooling inventory model.
            - Only one of O(cert_expiry) or O(cert_lifetime) may be specified.
        type: str
    cert_lifetime:
        description:
            - The lifetime of the certificate.
            - Applies to all certificates for accounts with a non-pooling inventory model.
            - O(cert_lifetime) is only supported for requests of O(request_type=new) or O(request_type=renew). If O(request_type=reissue), O(cert_lifetime) will
              be used for the first certificate issuance, but subsequent issuances will have the same expiry as the initial certificate.
            - Applies to certificates of O(cert_type=CDS_INDIVIDUAL), V(CDS_GROUP), V(CDS_ENT_LITE), V(CDS_ENT_PRO), or V(SMIME_ENT)
              for accounts with a pooling inventory model.
            - V(P1Y) is a certificate with a 1 year lifetime.
            - V(P2Y) is a certificate with a 2 year lifetime.
            - V(P3Y) is a certificate with a 3 year lifetime.
            - Only one of O(cert_expiry) or O(cert_lifetime) may be specified.
        type: str
        choices: [ P1Y, P2Y, P3Y ]
seealso:
    - module: community.crypto.openssl_privatekey
      description: Can be used to create private keys (both for certificates and accounts).
    - module: community.crypto.openssl_csr
      description: Can be used to create a Certificate Signing Request (CSR).
    - plugin: community.crypto.to_serial
      plugin_type: filter
'''

EXAMPLES = r'''
- name: Request a new certificate from Entrust with bare minimum parameters.
        Will request a new certificate if current one is valid but within 30
        days of expiry. If replacing an existing file in path, will back it up.
  community.crypto.ecs_certificate:
    backup: true
    path: /etc/ssl/crt/ansible.com.crt
    full_chain_path: /etc/ssl/crt/ansible.com.chain.crt
    csr: /etc/ssl/csr/ansible.com.csr
    cert_type: EV_SSL
    requester_name: Jo Doe
    requester_email: jdoe@ansible.com
    requester_phone: 555-555-5555
    entrust_api_user: apiusername
    entrust_api_key: a^lv*32!cd9LnT
    entrust_api_client_cert_path: /etc/ssl/entrust/ecs-client.crt
    entrust_api_client_cert_key_path: /etc/ssl/entrust/ecs-client.key

- name: If there is no certificate present in path, request a new certificate
        of type EV_SSL. Otherwise, if there is an Entrust managed certificate
        in path and it is within 63 days of expiration, request a renew of that
        certificate.
  community.crypto.ecs_certificate:
    path: /etc/ssl/crt/ansible.com.crt
    csr: /etc/ssl/csr/ansible.com.csr
    cert_type: EV_SSL
    cert_expiry: '2020-08-20'
    request_type: renew
    remaining_days: 63
    requester_name: Jo Doe
    requester_email: jdoe@ansible.com
    requester_phone: 555-555-5555
    entrust_api_user: apiusername
    entrust_api_key: a^lv*32!cd9LnT
    entrust_api_client_cert_path: /etc/ssl/entrust/ecs-client.crt
    entrust_api_client_cert_key_path: /etc/ssl/entrust/ecs-client.key

- name: If there is no certificate present in path, download certificate
        specified by tracking_id if it is still valid. Otherwise, if the
        certificate is within 79 days of expiration, request a renew of that
        certificate and save it in path. This can be used to "migrate" a
        certificate to be Ansible managed.
  community.crypto.ecs_certificate:
    path: /etc/ssl/crt/ansible.com.crt
    csr: /etc/ssl/csr/ansible.com.csr
    tracking_id: 2378915
    request_type: renew
    remaining_days: 79
    entrust_api_user: apiusername
    entrust_api_key: a^lv*32!cd9LnT
    entrust_api_client_cert_path: /etc/ssl/entrust/ecs-client.crt
    entrust_api_client_cert_key_path: /etc/ssl/entrust/ecs-client.key

- name: Force a reissue of the certificate specified by tracking_id.
  community.crypto.ecs_certificate:
    path: /etc/ssl/crt/ansible.com.crt
    force: true
    tracking_id: 2378915
    request_type: reissue
    entrust_api_user: apiusername
    entrust_api_key: a^lv*32!cd9LnT
    entrust_api_client_cert_path: /etc/ssl/entrust/ecs-client.crt
    entrust_api_client_cert_key_path: /etc/ssl/entrust/ecs-client.key

- name: Request a new certificate with an alternative client. Note that the
        issued certificate will have it's Subject Distinguished Name use the
        organization details associated with that client, rather than what is
        in the CSR.
  community.crypto.ecs_certificate:
    path: /etc/ssl/crt/ansible.com.crt
    csr: /etc/ssl/csr/ansible.com.csr
    client_id: 2
    requester_name: Jo Doe
    requester_email: jdoe@ansible.com
    requester_phone: 555-555-5555
    entrust_api_user: apiusername
    entrust_api_key: a^lv*32!cd9LnT
    entrust_api_client_cert_path: /etc/ssl/entrust/ecs-client.crt
    entrust_api_client_cert_key_path: /etc/ssl/entrust/ecs-client.key

- name: Request a new certificate with a number of CSR parameters overridden
        and tracking information
  community.crypto.ecs_certificate:
    path: /etc/ssl/crt/ansible.com.crt
    full_chain_path: /etc/ssl/crt/ansible.com.chain.crt
    csr: /etc/ssl/csr/ansible.com.csr
    subject_alt_name:
      - ansible.testcertificates.com
      - www.testcertificates.com
    eku: SERVER_AND_CLIENT_AUTH
    ct_log: true
    org: Test Organization Inc.
    ou:
      - Administration
    tracking_info: "Submitted via Ansible"
    additional_emails:
      - itsupport@testcertificates.com
      - jsmith@ansible.com
    custom_fields:
      text1: Admin
      text2: Invoice 25
      number1: 342
      date1: '2018-01-01'
      email1: sales@ansible.testcertificates.com
      dropdown1: red
    cert_expiry: '2020-08-15'
    requester_name: Jo Doe
    requester_email: jdoe@ansible.com
    requester_phone: 555-555-5555
    entrust_api_user: apiusername
    entrust_api_key: a^lv*32!cd9LnT
    entrust_api_client_cert_path: /etc/ssl/entrust/ecs-client.crt
    entrust_api_client_cert_key_path: /etc/ssl/entrust/ecs-client.key

'''

RETURN = '''
filename:
    description: The destination path for the generated certificate.
    returned: changed or success
    type: str
    sample: /etc/ssl/crt/www.ansible.com.crt
backup_file:
    description: Name of backup file created for the certificate.
    returned: changed and if O(backup) is V(true)
    type: str
    sample: /path/to/www.ansible.com.crt.2019-03-09@11:22~
backup_full_chain_file:
    description: Name of the backup file created for the certificate chain.
    returned: changed and if O(backup) is V(true) and O(full_chain_path) is set.
    type: str
    sample: /path/to/ca.chain.crt.2019-03-09@11:22~
tracking_id:
    description: The tracking ID to reference and track the certificate in ECS.
    returned: success
    type: int
    sample: 380079
serial_number:
    description:
        - The serial number of the issued certificate.
        - This return value is an B(integer). If you need the serial numbers as a colon-separated hex string,
          such as C(11:22:33), you need to convert it to that form with P(community.crypto.to_serial#filter).
    returned: success
    type: int
    sample: 1235262234164342
cert_days:
    description: The number of days the certificate remains valid.
    returned: success
    type: int
    sample: 253
cert_status:
    description:
        - The certificate status in ECS.
        - 'Current possible values (which may be expanded in the future) are: V(ACTIVE), V(APPROVED), V(DEACTIVATED), V(DECLINED), V(EXPIRED), V(NA),
          V(PENDING), V(PENDING_QUORUM), V(READY), V(REISSUED), V(REISSUING), V(RENEWED), V(RENEWING), V(REVOKED), V(SUSPENDED)'
    returned: success
    type: str
    sample: ACTIVE
cert_details:
    description:
        - The full response JSON from the Get Certificate call of the ECS API.
        - 'While the response contents are guaranteed to be forwards compatible with new ECS API releases, Entrust recommends that you do not make any
          playbooks take actions based on the content of this field. However it may be useful for debugging, logging, or auditing purposes.'
    returned: success
    type: dict

'''

from ansible_collections.community.crypto.plugins.module_utils.ecs.api import (
    ecs_client_argument_spec,
    ECSClient,
    RestOperationException,
    SessionConfigurationException,
)

import datetime
import os
import re
import time
import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_native, to_bytes

from ansible_collections.community.crypto.plugins.module_utils.version import LooseVersion

from ansible_collections.community.crypto.plugins.module_utils.io import (
    write_file,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.support import (
    load_certificate,
)

CRYPTOGRAPHY_IMP_ERR = None
try:
    import cryptography
    CRYPTOGRAPHY_VERSION = LooseVersion(cryptography.__version__)
except ImportError:
    CRYPTOGRAPHY_IMP_ERR = traceback.format_exc()
    CRYPTOGRAPHY_FOUND = False
else:
    CRYPTOGRAPHY_FOUND = True

MINIMAL_CRYPTOGRAPHY_VERSION = '1.6'


def validate_cert_expiry(cert_expiry):
    search_string_partial = re.compile(r'^([0-9]+)-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01])\Z')
    search_string_full = re.compile(r'^([0-9]+)-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01])[Tt]([01][0-9]|2[0-3]):([0-5][0-9]):'
                                    r'([0-5][0-9]|60)(.[0-9]+)?(([Zz])|([+|-]([01][0-9]|2[0-3]):[0-5][0-9]))\Z')
    if search_string_partial.match(cert_expiry) or search_string_full.match(cert_expiry):
        return True
    return False


def calculate_cert_days(expires_after):
    cert_days = 0
    if expires_after:
        expires_after_datetime = datetime.datetime.strptime(expires_after, '%Y-%m-%dT%H:%M:%SZ')
        cert_days = (expires_after_datetime - datetime.datetime.now()).days
    return cert_days


# Populate the value of body[dict_param_name] with the JSON equivalent of
# module parameter of param_name if that parameter is present, otherwise leave field
# out of resulting dict
def convert_module_param_to_json_bool(module, dict_param_name, param_name):
    body = {}
    if module.params[param_name] is not None:
        if module.params[param_name]:
            body[dict_param_name] = 'true'
        else:
            body[dict_param_name] = 'false'
    return body


class EcsCertificate(object):
    '''
    Entrust Certificate Services certificate class.
    '''

    def __init__(self, module):
        self.path = module.params['path']
        self.full_chain_path = module.params['full_chain_path']
        self.force = module.params['force']
        self.backup = module.params['backup']
        self.request_type = module.params['request_type']
        self.csr = module.params['csr']

        # All return values
        self.changed = False
        self.filename = None
        self.tracking_id = None
        self.cert_status = None
        self.serial_number = None
        self.cert_days = None
        self.cert_details = None
        self.backup_file = None
        self.backup_full_chain_file = None

        self.cert = None
        self.ecs_client = None
        if self.path and os.path.exists(self.path):
            try:
                self.cert = load_certificate(self.path, backend='cryptography')
            except Exception as dummy:
                self.cert = None
        # Instantiate the ECS client and then try a no-op connection to verify credentials are valid
        try:
            self.ecs_client = ECSClient(
                entrust_api_user=module.params['entrust_api_user'],
                entrust_api_key=module.params['entrust_api_key'],
                entrust_api_cert=module.params['entrust_api_client_cert_path'],
                entrust_api_cert_key=module.params['entrust_api_client_cert_key_path'],
                entrust_api_specification_path=module.params['entrust_api_specification_path']
            )
        except SessionConfigurationException as e:
            module.fail_json(msg='Failed to initialize Entrust Provider: {0}'.format(to_native(e)))
        try:
            self.ecs_client.GetAppVersion()
        except RestOperationException as e:
            module.fail_json(msg='Please verify credential information. Received exception when testing ECS connection: {0}'.format(to_native(e.message)))

    # Conversion of the fields that go into the 'tracking' parameter of the request object
    def convert_tracking_params(self, module):
        body = {}
        tracking = {}
        if module.params['requester_name']:
            tracking['requesterName'] = module.params['requester_name']
        if module.params['requester_email']:
            tracking['requesterEmail'] = module.params['requester_email']
        if module.params['requester_phone']:
            tracking['requesterPhone'] = module.params['requester_phone']
        if module.params['tracking_info']:
            tracking['trackingInfo'] = module.params['tracking_info']
        if module.params['custom_fields']:
            # Omit custom fields from submitted dict if not present, instead of submitting them with value of 'null'
            # The ECS API does technically accept null without error, but it complicates debugging user escalations and is unnecessary bandwidth.
            custom_fields = {}
            for k, v in module.params['custom_fields'].items():
                if v is not None:
                    custom_fields[k] = v
            tracking['customFields'] = custom_fields
        if module.params['additional_emails']:
            tracking['additionalEmails'] = module.params['additional_emails']
        body['tracking'] = tracking
        return body

    def convert_cert_subject_params(self, module):
        body = {}
        if module.params['subject_alt_name']:
            body['subjectAltName'] = module.params['subject_alt_name']
        if module.params['org']:
            body['org'] = module.params['org']
        if module.params['ou']:
            body['ou'] = module.params['ou']
        return body

    def convert_general_params(self, module):
        body = {}
        if module.params['eku']:
            body['eku'] = module.params['eku']
        if self.request_type == 'new':
            body['certType'] = module.params['cert_type']
        body['clientId'] = module.params['client_id']
        body.update(convert_module_param_to_json_bool(module, 'ctLog', 'ct_log'))
        body.update(convert_module_param_to_json_bool(module, 'endUserKeyStorageAgreement', 'end_user_key_storage_agreement'))
        return body

    def convert_expiry_params(self, module):
        body = {}
        if module.params['cert_lifetime']:
            body['certLifetime'] = module.params['cert_lifetime']
        elif module.params['cert_expiry']:
            body['certExpiryDate'] = module.params['cert_expiry']
        # If neither cerTLifetime or certExpiryDate was specified and the request type is new, default to 365 days
        elif self.request_type != 'reissue':
            gmt_now = datetime.datetime.fromtimestamp(time.mktime(time.gmtime()))
            expiry = gmt_now + datetime.timedelta(days=365)
            body['certExpiryDate'] = expiry.strftime("%Y-%m-%dT%H:%M:%S.00Z")
        return body

    def set_tracking_id_by_serial_number(self, module):
        try:
            # Use serial_number to identify if certificate is an Entrust Certificate
            # with an associated tracking ID
            serial_number = "{0:X}".format(self.cert.serial_number)
            cert_results = self.ecs_client.GetCertificates(serialNumber=serial_number).get('certificates', {})
            if len(cert_results) == 1:
                self.tracking_id = cert_results[0].get('trackingId')
        except RestOperationException as dummy:
            # If we fail to find a cert by serial number, that's fine, we just do not set self.tracking_id
            return

    def set_cert_details(self, module):
        try:
            self.cert_details = self.ecs_client.GetCertificate(trackingId=self.tracking_id)
            self.cert_status = self.cert_details.get('status')
            self.serial_number = self.cert_details.get('serialNumber')
            self.cert_days = calculate_cert_days(self.cert_details.get('expiresAfter'))
        except RestOperationException as e:
            module.fail_json('Failed to get details of certificate with tracking_id="{0}", Error: '.format(self.tracking_id), to_native(e.message))

    def check(self, module):
        if self.cert:
            # We will only treat a certificate as valid if it is found as a managed entrust cert.
            # We will only set updated tracking ID based on certificate in "path" if it is managed by entrust.
            self.set_tracking_id_by_serial_number(module)

            if module.params['tracking_id'] and self.tracking_id and module.params['tracking_id'] != self.tracking_id:
                module.warn('tracking_id parameter of "{0}" provided, but will be ignored. Valid certificate was present in path "{1}" with '
                            'tracking_id of "{2}".'.format(module.params['tracking_id'], self.path, self.tracking_id))

        # If we did not end up setting tracking_id based on existing cert, get from module params
        if not self.tracking_id:
            self.tracking_id = module.params['tracking_id']

        if not self.tracking_id:
            return False

        self.set_cert_details(module)

        if self.cert_status == 'EXPIRED' or self.cert_status == 'SUSPENDED' or self.cert_status == 'REVOKED':
            return False
        if self.cert_days < module.params['remaining_days']:
            return False

        return True

    def request_cert(self, module):
        if not self.check(module) or self.force:
            body = {}

            # Read the CSR contents
            if self.csr and os.path.exists(self.csr):
                with open(self.csr, 'r') as csr_file:
                    body['csr'] = csr_file.read()

            # Check if the path is already a cert
            # tracking_id may be set as a parameter or by get_cert_details if an entrust cert is in 'path'. If tracking ID is null
            # We will be performing a reissue operation.
            if self.request_type != 'new' and not self.tracking_id:
                module.warn('No existing Entrust certificate found in path={0} and no tracking_id was provided, setting request_type to "new" for this task'
                            'run. Future playbook runs that point to the pathination file in {1} will use request_type={2}'
                            .format(self.path, self.path, self.request_type))
                self.request_type = 'new'
            elif self.request_type == 'new' and self.tracking_id:
                module.warn('Existing certificate being acted upon, but request_type is "new", so will be a new certificate issuance rather than a'
                            'reissue or renew')
            # Use cases where request type is new and no existing certificate, or where request type is reissue/renew and a valid
            # existing certificate is found, do not need warnings.

            body.update(self.convert_tracking_params(module))
            body.update(self.convert_cert_subject_params(module))
            body.update(self.convert_general_params(module))
            body.update(self.convert_expiry_params(module))

            if not module.check_mode:
                try:
                    if self.request_type == 'validate_only':
                        body['validateOnly'] = 'true'
                        result = self.ecs_client.NewCertRequest(Body=body)
                    if self.request_type == 'new':
                        result = self.ecs_client.NewCertRequest(Body=body)
                    elif self.request_type == 'renew':
                        result = self.ecs_client.RenewCertRequest(trackingId=self.tracking_id, Body=body)
                    elif self.request_type == 'reissue':
                        result = self.ecs_client.ReissueCertRequest(trackingId=self.tracking_id, Body=body)
                    self.tracking_id = result.get('trackingId')
                    self.set_cert_details(module)
                except RestOperationException as e:
                    module.fail_json(msg='Failed to request new certificate from Entrust (ECS) {0}'.format(e.message))

                if self.request_type != 'validate_only':
                    if self.backup:
                        self.backup_file = module.backup_local(self.path)
                    write_file(module, to_bytes(self.cert_details.get('endEntityCert')))
                    if self.full_chain_path and self.cert_details.get('chainCerts'):
                        if self.backup:
                            self.backup_full_chain_file = module.backup_local(self.full_chain_path)
                        chain_string = '\n'.join(self.cert_details.get('chainCerts')) + '\n'
                        write_file(module, to_bytes(chain_string), path=self.full_chain_path)
                    self.changed = True
        # If there is no certificate present in path but a tracking ID was specified, save it to disk
        elif not os.path.exists(self.path) and self.tracking_id:
            if not module.check_mode:
                write_file(module, to_bytes(self.cert_details.get('endEntityCert')))
                if self.full_chain_path and self.cert_details.get('chainCerts'):
                    chain_string = '\n'.join(self.cert_details.get('chainCerts')) + '\n'
                    write_file(module, to_bytes(chain_string), path=self.full_chain_path)
            self.changed = True

    def dump(self):
        result = {
            'changed': self.changed,
            'filename': self.path,
            'tracking_id': self.tracking_id,
            'cert_status': self.cert_status,
            'serial_number': self.serial_number,
            'cert_days': self.cert_days,
            'cert_details': self.cert_details,
        }
        if self.backup_file:
            result['backup_file'] = self.backup_file
            result['backup_full_chain_file'] = self.backup_full_chain_file
        return result


def custom_fields_spec():
    return dict(
        text1=dict(type='str'),
        text2=dict(type='str'),
        text3=dict(type='str'),
        text4=dict(type='str'),
        text5=dict(type='str'),
        text6=dict(type='str'),
        text7=dict(type='str'),
        text8=dict(type='str'),
        text9=dict(type='str'),
        text10=dict(type='str'),
        text11=dict(type='str'),
        text12=dict(type='str'),
        text13=dict(type='str'),
        text14=dict(type='str'),
        text15=dict(type='str'),
        number1=dict(type='float'),
        number2=dict(type='float'),
        number3=dict(type='float'),
        number4=dict(type='float'),
        number5=dict(type='float'),
        date1=dict(type='str'),
        date2=dict(type='str'),
        date3=dict(type='str'),
        date4=dict(type='str'),
        date5=dict(type='str'),
        email1=dict(type='str'),
        email2=dict(type='str'),
        email3=dict(type='str'),
        email4=dict(type='str'),
        email5=dict(type='str'),
        dropdown1=dict(type='str'),
        dropdown2=dict(type='str'),
        dropdown3=dict(type='str'),
        dropdown4=dict(type='str'),
        dropdown5=dict(type='str'),
    )


def ecs_certificate_argument_spec():
    return dict(
        backup=dict(type='bool', default=False),
        force=dict(type='bool', default=False),
        path=dict(type='path', required=True),
        full_chain_path=dict(type='path'),
        tracking_id=dict(type='int'),
        remaining_days=dict(type='int', default=30),
        request_type=dict(type='str', default='new', choices=['new', 'renew', 'reissue', 'validate_only']),
        cert_type=dict(type='str', choices=['STANDARD_SSL',
                                            'ADVANTAGE_SSL',
                                            'UC_SSL',
                                            'EV_SSL',
                                            'WILDCARD_SSL',
                                            'PRIVATE_SSL',
                                            'PD_SSL',
                                            'CODE_SIGNING',
                                            'EV_CODE_SIGNING',
                                            'CDS_INDIVIDUAL',
                                            'CDS_GROUP',
                                            'CDS_ENT_LITE',
                                            'CDS_ENT_PRO',
                                            'SMIME_ENT',
                                            ]),
        csr=dict(type='str'),
        subject_alt_name=dict(type='list', elements='str'),
        eku=dict(type='str', choices=['SERVER_AUTH', 'CLIENT_AUTH', 'SERVER_AND_CLIENT_AUTH']),
        ct_log=dict(type='bool'),
        client_id=dict(type='int', default=1),
        org=dict(type='str'),
        ou=dict(type='list', elements='str'),
        end_user_key_storage_agreement=dict(type='bool'),
        tracking_info=dict(type='str'),
        requester_name=dict(type='str', required=True),
        requester_email=dict(type='str', required=True),
        requester_phone=dict(type='str', required=True),
        additional_emails=dict(type='list', elements='str'),
        custom_fields=dict(type='dict', default=None, options=custom_fields_spec()),
        cert_expiry=dict(type='str'),
        cert_lifetime=dict(type='str', choices=['P1Y', 'P2Y', 'P3Y']),
    )


def main():
    ecs_argument_spec = ecs_client_argument_spec()
    ecs_argument_spec.update(ecs_certificate_argument_spec())
    module = AnsibleModule(
        argument_spec=ecs_argument_spec,
        required_if=(
            ['request_type', 'new', ['cert_type']],
            ['request_type', 'validate_only', ['cert_type']],
            ['cert_type', 'CODE_SIGNING', ['end_user_key_storage_agreement']],
            ['cert_type', 'EV_CODE_SIGNING', ['end_user_key_storage_agreement']],
        ),
        mutually_exclusive=(
            ['cert_expiry', 'cert_lifetime'],
        ),
        supports_check_mode=True,
    )

    if not CRYPTOGRAPHY_FOUND or CRYPTOGRAPHY_VERSION < LooseVersion(MINIMAL_CRYPTOGRAPHY_VERSION):
        module.fail_json(msg=missing_required_lib('cryptography >= {0}'.format(MINIMAL_CRYPTOGRAPHY_VERSION)),
                         exception=CRYPTOGRAPHY_IMP_ERR)

    # If validate_only is used, pointing to an existing tracking_id is an invalid operation
    if module.params['tracking_id']:
        if module.params['request_type'] == 'new' or module.params['request_type'] == 'validate_only':
            module.fail_json(msg='The tracking_id field is invalid when request_type="{0}".'.format(module.params['request_type']))

    # A reissued request can not specify an expiration date or lifetime
    if module.params['request_type'] == 'reissue':
        if module.params['cert_expiry']:
            module.fail_json(msg='The cert_expiry field is invalid when request_type="reissue".')
        elif module.params['cert_lifetime']:
            module.fail_json(msg='The cert_lifetime field is invalid when request_type="reissue".')
    # Only a reissued request can omit the CSR
    else:
        module_params_csr = module.params['csr']
        if module_params_csr is None:
            module.fail_json(msg='The csr field is required when request_type={0}'.format(module.params['request_type']))
        elif not os.path.exists(module_params_csr):
            module.fail_json(msg='The csr field of {0} was not a valid path. csr is required when request_type={1}'.format(
                module_params_csr, module.params['request_type']))

    if module.params['ou'] and len(module.params['ou']) > 1:
        module.fail_json(msg='Multiple "ou" values are not currently supported.')

    if module.params['end_user_key_storage_agreement']:
        if module.params['cert_type'] != 'CODE_SIGNING' and module.params['cert_type'] != 'EV_CODE_SIGNING':
            module.fail_json(msg='Parameter "end_user_key_storage_agreement" is valid only for cert_types "CODE_SIGNING" and "EV_CODE_SIGNING"')

    if module.params['org'] and module.params['client_id'] != 1 and module.params['cert_type'] != 'PD_SSL':
        module.fail_json(msg='The "org" parameter is not supported when client_id parameter is set to a value other than 1, unless cert_type is "PD_SSL".')

    if module.params['cert_expiry']:
        if not validate_cert_expiry(module.params['cert_expiry']):
            module.fail_json(msg='The "cert_expiry" parameter of "{0}" is not a valid date or date-time'.format(module.params['cert_expiry']))

    certificate = EcsCertificate(module)
    certificate.request_cert(module)
    result = certificate.dump()
    module.exit_json(**result)


if __name__ == '__main__':
    main()
